// Copyright (c) 2009 Steve Gribble.
//
// linked_list.h:   a simple linked list implementation
//
// Note that this linked list implementation is *not* thread-safe!

#ifndef linked_list_h_
#define linked_list_h_

// Some typedefs for the linked list and its elements.
typedef void *linked_list_t;
typedef void *list_element_t;
typedef int boolean_t;

#ifndef FALSE
#define FALSE 0
#endif  // FALSE

#ifndef TRUE
#define TRUE 1
#endif  // TRUE


// Create and return a new linked_list.  Returns NULL on
// failure.
extern linked_list_t linked_list_create();

// Destroy a previously created linked list.  Note that this
// will *not* attempt to free any of the linked list elements
// stored within the list.
extern void linked_list_destroy(linked_list_t list);

// Add an element to the front of the linked list.  Returns
// TRUE on success, FALSE on failure.
extern boolean_t linked_list_add_to_front(linked_list_t list,
                                          list_element_t element);

// Add an element to the back of the linked list.  Returns TRUE
// on success, FALSE on failure.
extern boolean_t linked_list_add_to_back(linked_list_t list,
                                         list_element_t element);

// Remove an element from the front of the linked list.
extern void linked_list_remove_from_front(linked_list_t list);

// Remove an element from the back of the linked list.
extern void linked_list_remove_from_back(linked_list_t list);

// Return the front of the list; doesn't remove it from the list.
extern list_element_t linked_list_front(linked_list_t list);

// Return the back of the list; doesn't remove it from the list.
extern list_element_t linked_list_back(linked_list_t list);

// Is the list empty?  If so, return TRUE, else return FALSE.
extern boolean_t is_list_empty(linked_list_t list);

#endif linked_list_h_
